@@ -5,35 +5,41 @@ Utilities for reading and writing Mach-O headers
 from pkg_resources import require
 require("altgraph")
 
+
 import sys
 import struct
 
-from altgraph.compat import *
-
 from macholib.mach_o import *
 from macholib.dyld import dyld_find, framework_info
 from macholib.util import fileview
+from macholib._compat import B
+try:
+    from macholib.compat import bytes
+except ImportError:
+    pass
 
 __all__ = ['MachO']
 
-RELOCATABLE = set((
+_RELOCATABLE = set((
     # relocatable commands that should be used for dependency walking
     LC_LOAD_DYLIB,
     LC_LOAD_WEAK_DYLIB,
     LC_PREBOUND_DYLIB,
+    LC_REEXPORT_DYLIB,
 ))
 
-RELOCATABLE_NAMES = {
+_RELOCATABLE_NAMES = {
     LC_LOAD_DYLIB: 'load_dylib',
     LC_LOAD_WEAK_DYLIB: 'load_weak_dylib',
     LC_PREBOUND_DYLIB: 'prebound_dylib',
+    LC_REEXPORT_DYLIB: 'reexport_dylib',
 }
 
-def shouldRelocateCommand(cmd):
+def _shouldRelocateCommand(cmd):
     """
     Should this command id be investigated for relocation?
     """
-    return cmd in RELOCATABLE
+    return cmd in _RELOCATABLE
 
 class MachO(object):
     """
@@ -58,7 +64,7 @@ class MachO(object):
         # initialized by load
         self.fat = None
         self.headers = []
-        self.load(file(filename, 'rb'))
+        self.load(open(filename, 'rb'))
 
     def __repr__(self):
         return "<MachO filename=%r>" % (self.filename,)
@@ -238,16 +244,17 @@ class MachOHeader(object):
             raise ValueError("total_size > low_offset (%d > %d)" % (
                 self.total_size, low_offset))
 
-    def walkRelocatables(self, shouldRelocateCommand=shouldRelocateCommand):
+    def walkRelocatables(self, shouldRelocateCommand=_shouldRelocateCommand):
         """
         for all relocatable commands
         yield (command_index, command_name, filename)
         """
         for (idx, (lc, cmd, data)) in enumerate(self.commands):
             if shouldRelocateCommand(lc.cmd):
-                name = RELOCATABLE_NAMES[lc.cmd]
+                name = _RELOCATABLE_NAMES[lc.cmd]
                 ofs = cmd.name - sizeof(lc.__class__) - sizeof(cmd.__class__)
-                yield idx, name, data[ofs:data.find('\x00', ofs)]
+                yield idx, name, data[ofs:data.find(B('\x00'), ofs)].decode(
+                        sys.getfilesystemencoding())
 
     def rewriteInstallNameCommand(self, loadcmd):
         """Rewrite the load command of this dylib"""
@@ -268,19 +275,22 @@ class MachOHeader(object):
         data = changefunc(self.parent.filename)
         changed = False
         if data is not None:
-            if self.rewriteInstallNameCommand(data):
+            if self.rewriteInstallNameCommand(
+                    data.encode(sys.getfilesystemencoding())):
                 changed = True
         for idx, name, filename in self.walkRelocatables():
             data = changefunc(filename)
             if data is not None:
-                if self.rewriteDataForCommand(idx, data):
+                if self.rewriteDataForCommand(idx, data.encode(
+                        sys.getfilesystemencoding())):
                     changed = True
         return changed
 
     def rewriteDataForCommand(self, idx, data):
         lc, cmd, old_data = self.commands[idx]
         hdrsize = sizeof(lc.__class__) + sizeof(cmd.__class__)
-        data = data + ('\x00' * (4 - (len(data) % 4)))
+        align = struct.calcsize('L')
+        data = data + (B('\x00') * (align - (len(data) % align)))
         newsize = hdrsize + len(data)
         self.commands[idx] = (lc, cmd, data)
         self.changedHeaderSizeBy(newsize - lc.cmdsize)
@@ -305,10 +315,11 @@ class MachOHeader(object):
         for lc, cmd, data in self.commands:
             lc.to_fileobj(fileobj)
             cmd.to_fileobj(fileobj)
+
             if isinstance(data, unicode):
-                data = data.encode('utf-8')
+                fileobj.write(data.encode(sys.getfilesystemencoding()))
             
-            if isinstance(data, str):
+            elif isinstance(data, (bytes, str)):
                 fileobj.write(data)
             else:
                 # segments..
@@ -317,7 +328,7 @@ class MachOHeader(object):
 
         # zero out the unused space, doubt this is strictly necessary
         # and is generally probably already the case
-        fileobj.write('\x00' * (self.low_offset - fileobj.tell()))
+        fileobj.write(B('\x00') * (self.low_offset - fileobj.tell()))
 
     def getSymbolTableCommand(self):
         for lc, cmd, data in self.commands:
@@ -340,7 +351,7 @@ def main(fn):
         for idx, name, other in header.walkRelocatables():
             if other not in seen:
                 seen.add(other)
-                print '\t' + other
+                print '\t' + name + ": " + other
 
 if __name__ == '__main__':
     import sys
